var app = angular.module('LineSidePrintingApp', []);

app.controller('LineSidePrintingController', ['$scope', '$compile', function ($scope, $compile) {

    toastr.options.positionClass = 'toast-bottom-full-width';
    toastr.options.tapToDismiss = false;
    var $toastrArray = [];


    $scope.LoggedIn = false;
    $scope.Version = "...";
    $scope.License = "...";

    $scope.shouldLogDebug = false;
    $scope.getLogDebug = function () {
        var response = { Message: "read-registry", Key: "AppData/LineSidePrinting/LogDebug" };
        webSocket.sendJson(response);
    };
    $scope.toggleLogDebug = function () {
        $scope.shouldLogDebug = !$scope.shouldLogDebug;
        var response = { Message: "write-registry", Key: "AppData/LineSidePrinting/LogDebug", Value: $scope.shouldLogDebug.toString() };
        webSocket.sendJson(response);
    };

    $scope.Config = {};

    $scope.Config.General = GeneralConfig;
    $scope.SavedGeneral = angular.copy($scope.Config.General);

    $scope.Config.InputConfig = InputConfig;
    $scope.SavedInputConfig = angular.copy($scope.Config.InputConfig);

    $scope.Config.BarcodeVerification = BarcodeVerificationConfig;
    $scope.SavedBarcodeVerification = angular.copy($scope.Config.BarcodeVerification);

    $scope.Config.LabelGeneration = LabelGenerationConfig;
    $scope.SavedLabelGeneration = angular.copy($scope.Config.LabelGeneration);

    $scope.Labels = [];
    $scope.SelectedLabel = "";



    $scope.label = new Label();
    $scope.SavedLabel = angular.copy($scope.label);

    $scope.applicationLog = "Loading...";
    $scope.CurrentInputStates = 0;


    $('#navbar a[href="#application-log-tab"]').click(function () {
        $.ajax({
            url: "log.php?filename=linesideprinting.log",
            context: document.body
        }).done(function (data) {
            $scope.applicationLog = data;
            $scope.$apply();

            setTimeout(function () {
                var docHeight = $(document).height();
                $("html, body").animate({ scrollTop: docHeight }, 500);
            }, 1);
        });
    });



    var saveResponse = null;
    var loginDialog = null;
    var enterLicenseDialog = null;
    var printPreviewDialog = null;

    $('#loading-info').html("Create websocket connection");


    // create our websocket connection
    var webSocket = new JnrWebSocket({
        defaultIpAddress: "10.0.0.43",
        appId: 5002,
        onopen: function (evt) {
            webSocket.messageHandlers['Error'] = error;
            webSocket.messageHandlers['License-Status'] = licenseStatus;
            webSocket.messageHandlers['License'] = licenseInfo;
            webSocket.messageHandlers['config-resp'] = configResp;
            webSocket.messageHandlers['input-states'] = updateInputStates;
            webSocket.messageHandlers['status-update'] = statusUpdate;
            webSocket.messageHandlers['print-response'] = $scope.printResponse;
            webSocket.messageHandlers['barcode-scan'] = $scope.barcodeScan;
            webSocket.messageHandlers['load-part-file-response'] = commandSuccessFailResponse;
            webSocket.messageHandlers['part-file-contents'] = showPartFileContents;
            webSocket.messageHandlers['read-registry-resp'] = readRegistryResponse;
        },
        getAuthentication: function (evt, jobj) {
            getCredentials(evt, jobj);

            var activeTab = $('.tab-pane.active');
            activeTab.removeClass('active');

            var $homeTab = $('#home-tab');
            $homeTab.addClass("active");
        },
        authenticated: function (evt, jobj) {
            $scope.LoggedIn = true;
            $scope.getConfig();

            loginDialog.modal('hide');

            //            if ($toastrArray['login'] != null) {
            //                $toastrArray['login'].attr('class', 'toast toast-success');
            //                $toastrArray['login'].html('User logged in');
            //                setTimeout(function () {
            //                    toastr.clear($toastrArray['login']);
            //                    $toastrArray['login'] = null;
            //                }, 1000);
            //            }
        }

    });
    webSocket.connect();



    var readRegistryResponse = function (evt, jobj) {
        var key = jobj['Key'];
        var value = jobj['Value'];
        console.log(key + " = " + value);

        if (key.indexOf("LogDebug") > 0)
            $scope.shouldLogDebug = (value && "true" == value);

        $scope.$apply();
    };



    function getCredentials(evt, jobj) {
        var modalAlreadyOpen = $('#login-dialog').hasClass('in');
        if (!modalAlreadyOpen && enterLicenseDialog == undefined) {
            loginDialog = bootbox.dialog({
                closeButton: false,
                title: "Please enter your user credentials",
                message: '<form role="form">' +
                    '    <div class="modal-body">' +
                    '        <div class="form-group">' +
                    '            <label for="login-username">Username</label>' +
                    '            <input type="text" class="form-control" id="login-username" placeholder="Enter username" >' +
                    '        </div>' +
                    '        <div class="form-group">' +
                    '            <label for="login-password">Password</label>' +
                    '            <input type="password" class="form-control" id="login-password" placeholder="Enter password" >' +
                    '        </div>' +
                    '    </div>' +
                    '</form>',
                buttons: {
                    main: {
                        label: "Log In",
                        className: "btn-primary",
                        callback: function () {
                            $("#bootbox-main-btn").html("Logging In...");

                            setTimeout(function () {
                                var username = $('#login-username').val();
                                var password = $('#login-password').val();
                                $scope.login(username, password);
                                return false;
                            }, 1000);

                            return false;
                        }
                    }
                }
            });
            loginDialog.init(function () {
                $(".modal-header").css("background-color", "#428bca");
                $(".modal-header").css("color", "#fff");
            });
            loginDialog.attr('id', 'login-dialog');

            loginDialog.on("shown.bs.modal", function () {
                _loginSent = true;
                $('#login-username').focus();
            });

            loginDialog.keypress(function (e) {
                if (e.which == 13) {
                    $("#bootbox-main-btn").html("Logging In...");

                    setTimeout(function () {
                        var username = $('#login-username').val();
                        var password = $('#login-password').val();
                        $scope.login(username, password);
                        return false;
                    }, 1000);

                }
            });
        }
    }



    $scope.login = function (username, password) {
        //        if ($toastrArray['login'] != null) {
        //            $toastrArray['login'].attr('class', 'toast toast-info');
        //            $toastrArray['login'].html("Logging in as '" + username + "'...");
        //        } else
        //            $toastrArray['login'] = toastr.info("Logging in as '" + username + "'...", "", {timeOut: 0});

        var response = {};
        response.Message = 'login';
        response.Username = username;
        response.Password = password;
        webSocket.sendJson(response);

        _loginSent = true;
    };



    $scope.isLicenseValid = function () {
        return $scope.License.toLowerCase() === "valid";
    };



    $scope.enterLicenseKey = function () {
        if (!enterLicenseDialog) {
            enterLicenseDialog = bootbox.dialog({
                closeButton: false,
                title: "Enter License",
                message: '<form role="form">' +
                    '    <div class="modal-body">' +
                    '        <p>Enter the 16 digit license key with dashes</p>' +
                    '        <div class="form-group">' +
                    '            <input type="text" class="form-control" id="license-key" placeholder="Enter License Key" >' +
                    '        </div>' +
                    '    </div>' +
                    '</form>',
                buttons: {
                    submit: {
                        label: "Submit",
                        className: "btn-primary",
                        callback: function () {
                            if (!$scope.isLicenseValid()) {
                                submitLicense();
                                return false;
                            }
                        }
                    }
                }
            });
            enterLicenseDialog.init(function () {
                $(".modal-header").css("background-color", "#428bca");
                $(".modal-header").css("color", "#fff");
            });
            enterLicenseDialog.attr('id', 'enter-license-key-dialog');

            enterLicenseDialog.on("shown.bs.modal", function () {
                $('#license-key').focus();
            });

            var submitLicense = function () {
                $("#bootbox-submit-btn").html("Validating...");

                setTimeout(function () {
                    var licenseKey = $('#license-key').val();
                    submitLicenseKey(licenseKey);
                }, 1000);
            };


            enterLicenseDialog.keypress(function (e) {
                if (e.which === 13) {
                    submitLicense();
                    return false;
                }
            });
        }
    };



    var submitLicenseKey = function (licenseKey) {
        var response = {};
        response.Message = 'submit-license-key';
        response.LicenseKey = licenseKey;
        webSocket.sendJson(response);
    };



    var licenseStatus = function (evt, jobj) {
        if (jobj['Status'].toLowerCase() != 'valid') {
            $scope.enterLicenseKey();
        }
    };



    var licenseInfo = function (evt, jobj) {
        var text = jobj["Text"];
        if (text.toLowerCase() == "valid") {
            $("#bootbox-modal-title").html("License Validated.  Thank you!");
            $(".modal-header").css("background-color", "#5cb85c");
            $("#bootbox-submit-btn").html("OK");
            $("#bootbox-submit-btn").attr('class', 'btn btn-success');

            webSocket.sendPing();
        } else {
            $("#bootbox-modal-title").html("License Invalid.  Please enter a valid license");
            $(".modal-header").css("background-color", "#c9302c");
            $("#bootbox-submit-btn").html("Submit");
        }

        $scope.License = text;
        $scope.$apply();
    };



    $scope.loadPartFile = function (evt) {
        var f = evt[0].files[0];

        if (f.size > 102400) {
            toastr.error("Selected file is too large");
            return;
        }

        if (f) {
            var dialog = bootbox.confirm({
                title: "Are you sure you want to load the selected file?",
                message: "Are you sure you want to load '" + f.name + "' as the new part file?",
                buttons: {
                    cancel: {
                        label: 'No',
                        className: 'btn-default'
                    },
                    confirm: {
                        label: 'Yes',
                        className: 'btn-primary'
                    }
                },
                callback: function (result) {
                    if (result) {
                        var r = new FileReader();
                        r.onload = function (e) {
                            var contents = e.target.result;

                            $("#bootbox-confirm-btn").html("Uploading Part File...;")

                            try {
                                var msg = new Object();
                                msg.Message = "save-part-file";
                                msg.FileName = f.name;
                                msg.FileSize = f.size;
                                msg.Contents = contents;
                                webSocket.sendJson(msg);
                            } catch (e) {
                                alert("Not a valid configuration file");
                                return false;
                            }
                            return true;
                        };
                        r.readAsText(f);
                    }
                }
            });
            dialog.init(function () {
                $(".modal-header").css("background-color", "#428bca");
                $(".modal-header").css("color", "#fff");
            });
        }
    };



    $scope.showPartFile = function () {
        if ($toastrArray['command-in-progress'] != null) {
            $toastrArray['command-in-progress'].attr('class', 'toast toast-info');
            $toastrArray['command-in-progress'].html("Getting part file...");
        } else
            $toastrArray['command-in-progress'] = toastr.info("Getting part file...", "", { timeOut: 0 });

        var msg = { Message: 'get-part-file' };
        webSocket.sendJson(msg);
    };



    $scope.saveSettingsConfig = function () {
        var dialog = bootbox.confirm({
            title: "Are you sure you want to save your changes?",
            message: "Saving your configuration will take effect right away",
            buttons: {
                cancel: {
                    label: 'No',
                    className: 'btn-default'
                },
                confirm: {
                    label: 'Yes',
                    className: 'btn-success'
                }
            },
            callback: function (result) {
                if (result) {
                    if ($toastrArray['save-config'] != null) {
                        $toastrArray['save-config'].attr('class', 'toast toast-info');
                        $toastrArray['save-config'].html("Saving configuration...");
                    } else {
                        $toastrArray['save-config'] = toastr.info("Saving configuration...", "", { timeOut: 0 });
                    }

                    $("#bootbox-submit-btn").html("Saving...");

                    setTimeout(function () {
                        var msg = {};
                        msg.Message = 'save-settings-config';
                        msg.Config = {};
                        msg.Config.General = $scope.Config.General;
                        msg.Config.InputConfig = $scope.Config.InputConfig;
                        msg.Config.BarcodeVerification = $scope.Config.BarcodeVerification;
                        msg.Config.LabelGeneration = $scope.Config.LabelGeneration;
                        webSocket.sendJson(msg);
                    }, 100);
                }
            }
        });
        dialog.init(function () {
            $(".modal-header").css("background-color", "#5cb85c");
            $(".modal-header").css("color", "#fff");
        });
    };



    $scope.cancelSettingsConfig = function () {
        var dialog = bootbox.confirm({
            title: "Are you sure you want to cancel your changes?",
            message: "Cancelling your changes will reload the last saved settings",
            buttons: {
                cancel: {
                    label: 'No',
                    className: 'btn-default'
                },
                confirm: {
                    label: 'Yes',
                    className: 'btn-warning'
                }
            },
            callback: function (result) {
                if (result) {
                    $("#bootbox-confirm-btn").html("Requesting current configuration...");
                    $scope.getConfig();
                }
            }
        });
        dialog.init(function () {
            $(".modal-header").css("background-color", "#f0ad4e");
            $(".modal-header").css("color", "#fff");
        });
    };



    $scope.cancelChanges = function () {
        window.setTimeout(function () {
            getConfig();
        }, 500);
    }



    $scope.getConfig = function () {
        $('#loading-info').html("Getting Configuration");

        if ($toastrArray['getConfig'] != null) {
            $toastrArray['getConfig'].attr('class', 'toast toast-info');
            $toastrArray['getConfig'].html("Getting configuration...");
        } else
            $toastrArray['getConfig'] = toastr.info("Getting configuration...", "", { timeOut: 0 });

        setTimeout(function () {
            var json = new Object();
            json['Message'] = "get-config";
            webSocket.sendJson(json);
        }, 1000);
    }



    var error = function (evt, jobj) {
        if (jobj.Text == 'Login Failed') {
            if (_loginSent) {
                $("#bootbox-modal-title").html("Authentication Failed.  Please enter valid credentials");
                $(".modal-header").css("background-color", "#c9302c");
                $("#bootbox-main-btn").html("Log In");

                //                if ($toastrArray['login'] != null) {
                //                    $toastrArray['login'].attr('class', 'toast toast-error');
                //                    $toastrArray['login'].html('Either the Username or Password was incorrect');
                //                }
            }
        }
    };



    var configResp = function (evt, jobj) {

        console.log(jobj);

        $scope.Version = jobj.Version;
        $scope.License = jobj.License;
        if ($scope.License == undefined)
            $scope.License = "Invalid";

        if (jobj.Config != null && !isEmpty(jobj.Config)) {

            if (jobj.Config.General != undefined) {
                $scope.Config.HardwareConfig = jobj.Config.HardwareConfig;
            }

            if (jobj.Config.General != undefined) {
                $scope.Config.General = jobj.Config.General;
                $scope.SavedGeneral = angular.copy($scope.Config.General);
            }

            if (jobj.Config.InputConfig != undefined) {
                assignObject($scope.Config.InputConfig, jobj.Config.InputConfig);
                iterateObject($scope.Config.InputConfig);

                //                $scope.Config.InputConfig = jobj.Config.InputConfig;
                //                if ($scope.Config.InputConfig.Parts == undefined) {
                //                    $scope.Config.InputConfig.Parts = [];
                //                    for (var i = 0; i < $scope.Config.HardwareConfig.InputCount; i++) {
                //                        $scope.Config.InputConfig.Parts.push({State: false, PartNumber: "", Error: false, Status: ""});
                //                    }
                //                }
                $scope.SavedInputConfig = angular.copy($scope.Config.InputConfig);
            }

            if (jobj.Config.BarcodeVerification != undefined) {
                assignObject($scope.Config.BarcodeVerification, jobj.Config.BarcodeVerification);
                iterateObject($scope.Config.BarcodeVerification);

                //                $scope.Config.BarcodeVerification = jobj.Config.BarcodeVerification;
                $scope.SavedBarcodeVerification = angular.copy($scope.Config.BarcodeVerification);
            }

            if (jobj.Config.LabelGeneration != undefined) {
                assignObject($scope.Config.LabelGeneration, jobj.Config.LabelGeneration);
                iterateObject($scope.Config.LabelGeneration);

                //                $scope.Config.LabelGeneration = jobj.Config.LabelGeneration;
                $scope.SavedLabelGeneration = angular.copy($scope.Config.LabelGeneration);
            }

            if (jobj.Config.Labels != undefined) {
                $scope.Labels = jobj.Config.Labels;
                $scope.Labels.sort(function (a, b) { return a.toLowerCase().localeCompare(b.toLowerCase()); });
            }

            if (!isEmpty(jobj.Config.Label)) {
                $scope.SelectedLabel = jobj.Config.LabelName;
                for (var i = 0; i < jobj.Config.Label.items.length; i++) {
                    jobj.Config.Label.items[i].selected = false;
                }
                $scope.label.config = jobj.Config.Label;
                $scope.SavedLabel = angular.copy($scope.label);
                $scope.updateLabel();
            } else {
                $scope.SelectedLabel = '';
                $scope.label = new Label();
                $scope.SavedLabel = angular.copy($scope.label);
                $scope.updateLabel();
            }
            $scope.label.onchanged = $scope.updateLabel;
        }


        if ($toastrArray['getConfig'] != null) {
            $toastrArray['getConfig'].attr('class', 'toast toast-success');
            $toastrArray['getConfig'].html('Configuration Received');
            setTimeout(function () {
                toastr.clear($toastrArray['getConfig']);
                $toastrArray['getConfig'] = null;
            }, 1000);
        }

        if ($toastrArray['save-config'] != null) {
            $toastrArray['save-config'].attr('class', 'toast toast-success');
            $toastrArray['save-config'].html('Configuration saved Successfully');
            setTimeout(function () {
                toastr.clear($toastrArray['save-config']);
                $toastrArray['save-config'] = null;
            }, 1000);
        }

        if ($toastrArray['delete-label'] != null) {
            $toastrArray['delete-label'].attr('class', 'toast toast-success');
            $toastrArray['delete-label'].html('Label deleted successfully');
            setTimeout(function () {
                toastr.clear($toastrArray['delete-label']);
                $toastrArray['delete-label'] = null;
            }, 1000);
        }


        // we have made changes to our config.  apply them to the page.
        $scope.$apply();


        if (saveResponse != null) {
            saveResponse.resolve();
            saveResponse = null;
        }


        $('#blank-cover').hide();
    };



    function assignObject(object, newObject) {
        for (var property in object) {
            if (object.hasOwnProperty(property)) {
                console.log("current[" + property + "] = " + object[property] + ", new[" + property + "] = " + newObject[property]);
                if (typeof (object[property]) == 'object' && newObject[property] != undefined)
                    assignObject(object[property], newObject[property]);
                else {
                    var newValue = newObject[property];
                    if (newValue != undefined)
                        object[property] = newObject[property];
                }
            }
        }
    }



    function iterateObject(object) {
        for (var property in object) {
            if (object.hasOwnProperty(property)) {
                console.log(object + "." + property + " = " + object[property]);
                if (typeof (object[property]) == 'object')
                    iterateObject(object[property]);
            }
        }
    }



    function isEmpty(obj) {
        for (var prop in obj) {
            if (obj.hasOwnProperty(prop))
                return false;
        }

        return true;
    }



    var updateInputStates = function (evt, jobj) {
        $scope.CurrentInputStates = jobj['Value'];

        if ($scope.inputs != undefined) {
            for (var i = 0; i < $scope.inputs.length; i++) {
                $scope.inputs[i].state = ((($scope.CurrentInputStates >> i) & 1) == 1);
            }
        }

        $scope.$apply();
    }


    var statusUpdate = function (evt, jobj) {
        var inputNumber = jobj['InputNumber'];
        $scope.Config.InputConfig.Parts[inputNumber - 1].error = jobj['Error'] == 'true';
        $scope.Config.InputConfig.Parts[inputNumber - 1].status = new Date().toLocaleTimeString() + ' - ' + jobj['Text'];

        $scope.$apply();
    }



    var commandSuccessFailResponse = function (evt, jobj) {
        if (jobj['Result'] === 'success') {
            if ($toastrArray['command-in-progress'] != null) {
                $toastrArray['command-in-progress'].attr('class', 'toast toast-success');
                $toastrArray['command-in-progress'].html(jobj['Text']);
            } else
                toastr.success(jobj['Text'], "", { timeOut: 0 });

            setTimeout(function () {
                toastr.clear($toastrArray['command-in-progress']);
                $toastrArray['command-in-progress'] = null;
            }, 2000);
        } else {
            if ($toastrArray['command-in-progress'] != null) {
                $toastrArray['command-in-progress'].attr('class', 'toast toast-error');
                $toastrArray['command-in-progress'].html(jobj['Text'] + ' (This message will go away in 10 seconds)');
            } else
                $toastrArray['command-in-progress'] = toastr.error(jobj['Text'] + ' (This message will go away in 10 seconds)', "", { timeOut: 0 });

            setTimeout(function () {
                toastr.clear($toastrArray['command-in-progress']);
                $toastrArray['command-in-progress'] = null;
            }, 10000);
        }
    };



    var showPartFileContents = function (evt, jobj) {
        commandSuccessFailResponse(evt, jobj);

        if (jobj['Result'] === 'success') {
            var contents = '<div style="width: 100%; height: 450px; overflow: auto;">' + jobj['Contents'] + "</div>";

            var dialog = bootbox.alert({
                title: "Loaded Part File",
                message: "<pre style='background-color: #fff; border: 0;'>" + contents + "</pre>"
            });
            dialog.init(function () {
                $(".modal-header").css("background-color", "#428bca");
                $(".modal-header").css("color", "#fff");
            });

        }
    };



    $scope.newLabel = function () {
        bootbox.prompt("Enter the name of the new label", function (result) {
            if (result !== null) {
                if ($scope.Labels.indexOf(result) >= 0) {
                    alert("A label with this name already exists");
                    return false;
                }

                $scope.getLabel(result);
                $scope.$apply();
            }
        });
    };



    $scope.getLabel = function (labelName) {
        if ($scope.Labels.indexOf(labelName) >= 0) {
            var msg = { Message: 'get-config', Section: 'label', LabelName: labelName };
            webSocket.sendJson(msg);
        } else {
            $scope.label = new Label();
            $scope.SelectedLabel = labelName;
            $scope.SavedLabel = angular.copy($scope.label);
            $scope.updateLabel();
        }

        var navbar = $('#navbar');
        var activeNav = $('#navbar .active');
        activeNav.removeClass('active');
        var $labelPreviewNav = $('#label-preview-menu');
        $labelPreviewNav.addClass("active");

        var activeTab = $('.tab-pane.active');
        activeTab.removeClass('active');
        var $homeTab = $('#label-preview-tab');
        $homeTab.addClass("active");
    };



    $scope.saveLabel = function () {
        bootbox.confirm({
            title: "Are you sure you want to save your changes?",
            message: "Saving your configuration will take effect right away",
            callback: function (result) {
                if (result) {
                    if ($toastrArray['save-config'] != null) {
                        $toastrArray['save-config'].attr('class', 'toast toast-info');
                        $toastrArray['save-config'].html("Saving label configuration...");
                    } else
                        $toastrArray['save-config'] = toastr.info("Saving label configuration...", "", { timeOut: 0 });

                    setTimeout(function () {
                        var msg = { Message: 'save-label', LabelName: $scope.SelectedLabel, Label: $scope.label.config };
                        webSocket.sendJson(msg);
                        if ($scope.Labels.indexOf($scope.SelectedLabel) == -1) {
                            var msg = { Message: 'get-config', Section: 'labels' };
                            webSocket.sendJson(msg);
                        }
                    }, 1000);
                }
            }
        });
    };



    $scope.saveLabelAs = function () {
        bootbox.prompt({
            title: "Enter the label name",
            value: $scope.SelectedLabel,
            callback: function (result) {
                if (result !== null) {
                    if ($toastrArray['save-config'] != null) {
                        $toastrArray['save-config'].attr('class', 'toast toast-info');
                        $toastrArray['save-config'].html("Saving label configuration...");
                    } else
                        $toastrArray['save-config'] = toastr.info("Saving label configuration...", "", { timeOut: 0 });

                    setTimeout(function () {
                        var msg = { Message: 'save-label', LabelName: result, Label: $scope.label.config };
                        webSocket.sendJson(msg);
                    }, 1000);
                }
            }
        });
    };



    $scope.revertLabel = function () {
        bootbox.confirm({
            title: "Are you sure you want to revert your changes?",
            message: "Reverting your changes will reload the last saved settings.",
            callback: function (result) {
                if (result) {
                    if ($toastrArray['getConfig'] != null) {
                        $toastrArray['getConfig'].attr('class', 'toast toast-info');
                        $toastrArray['getConfig'].html("Getting configuration...");
                    } else
                        $toastrArray['getConfig'] = toastr.info("Getting configuration...", "", { timeOut: 0 });

                    $scope.getLabel($scope.SelectedLabel);
                }
            }
        });
    };



    $scope.hasLabelConfigChanged = function () {
        //        if ($scope.Labels.indexOf($scope.SelectedLabel) == -1)
        //            return false;
        return !angular.equals($scope.label, $scope.SavedLabel);
    };



    $scope.deleteLabel = function () {
        var message = "Are you sure you want to delete this label?";

        if ($scope.SelectedLabel == $scope.Config.LabelGeneration.ActiveLabel) {
            message += "<br><br><div style='color:red'>Warning: This is the Active Label</div>";
        }
        bootbox.confirm({
            title: "Are you sure you want to delete this label?",
            message: message,
            callback: function (result) {
                if (result) {
                    $toastrArray['delete-label'] = toastr.info('Deleting label', "", { timeOut: 0 });
                    webSocket.sendJson({ Message: 'delete-label', LabelName: $scope.SelectedLabel });
                    var $tabPane = $('.tab-pane');
                    $activeTabPane = $('.tab-pane.active');
                    alert($('.tab-pane.active').attr('id'));
                    $scope.getLabel('');
                }
            }
        });
    };



    $scope.updateLabel = function () {
        // if ($scope.label != null) {

        //     if ($scope.label != null)
        //         labelLayout($scope.label.config);

        //     var $labelPreviewElem = $("#label-preview");
        //     $labelPreviewElem.empty();
        //     for (var index in $scope.label.config.items) {
        //         var item = $scope.label.config.items[index];
        //         if (item != null) {
        //             if (item.type == 'Text')
        //                 showText(item);
        //             else if (item.type == 'Barcode')
        //                 showBarcode(item);
        //             else if (item.type == '2dBarcode')
        //                 show2dBarcode(item);
        //         }
        //     }

        //     var contents = $labelPreviewElem.contents();
        //     $compile(contents)($scope);
        //     // $scope.$apply();
        // }

        // $('[data-toggle="tooltip"]').tooltip({ container: 'body', html: true, template: '<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner" style="text-align:left; max-width:400px;"></div></div>' });
    };


    $scope.$watch('items', $scope.updateLabel(), true);


    $scope.printLabel = function () {

        printPreviewDialog = bootbox.dialog({
            closeButton: false,
            title: "Print Preview",
            message: '<form role="form">' +
                '    <div class="modal-body">' +
                '        <div class="form-group">' +
                '            <label for="print-preview-part-number">Part Number</label>' +
                '            <input type="text" class="form-control" id="print-preview-part-number" placeholder="Enter Part Number" value="ABC">' +
                '        </div>' +
                '        <div class="form-group">' +
                '            <label for="print-preview-sequence-number">Sequence Number</label>' +
                '            <input type="text" class="form-control" id="print-preview-sequence-number" placeholder="Enter Sequence Number" value="1">' +
                '        </div>' +
                '        <div class="form-group">' +
                '            <label for="print-preview-match-string">Match String</label>' +
                '            <input type="text" class="form-control" id="print-preview-match-string" placeholder="Enter Match String" value="">' +
                '        </div>' +
                '        <div class="form-group">' +
                '            <label for="print-preview-quantity">Quantity</label>' +
                '            <input type="text" class="form-control" id="print-preview-quantity" placeholder="Enter Quantity" value="1">' +
                '        </div>' +
                '    </div>' +
                '</form>',
            buttons: {
                submit: {
                    label: "Print",
                    className: "btn-primary",
                    callback: function () {
                        var partNumber = $('#print-preview-part-number').val();
                        var seqNumber = $('#print-preview-sequence-number').val();
                        var matchString = $('#print-preview-match-string').val();
                        var quantity = $('#print-preview-quantity').val();

                        $("#bootbox-submit-btn").hide();
                        $("#bootbox-cancel-btn").hide();

                        webSocket.sendJson({ 'Message': 'print', LabelName: $scope.SelectedLabel, 'Config': $scope.config, PartNo: partNumber, CustDate: "5917", Seq: seqNumber, MatchString: matchString, Qty: quantity });
                        return false;
                    }
                },
                cancel: {
                    label: "Cancel",
                    className: "btn-default"
                }
            }
        });
        printPreviewDialog.init(function () {
            $(".modal-header").css("background-color", "#428bca");
            $(".modal-header").css("color", "#fff");
        });
        printPreviewDialog.attr('id', 'print-preview-dialog');

        //        printPreviewDialog.keypress(function(e) {
        //            if (e.which == 13) {
        //                var partNumber = $('#print-preview-part-number').val();
        //                var seqNumber = $('#print-preview-sequence-number').val();
        //                var quantity = $('#print-preview-quantity').val();
        //                webSocket.sendJson({'Message': 'print', LabelName: $scope.SelectedLabel, 'Config': $scope.config, PartNo: partNumber, CustDate: "5917", Seq: seqNumber, Qty: quantity});
        //                printPreviewDialog.modal('hide');
        //            }
        //        });
    }



    $scope.printResponse = function (evt, jobj) {
        var $history = $("#history");

        var timestamp = jobj['Timestamp'];
        var date = new Date(timestamp);
        if (jobj['Result'] === 'success') {
            $history.prepend(moment(date).format('MMMM Do YYYY, h:mm:ss a') + ' : <span style="color:green">' + jobj['Text'] + '</span><br>');

            if ($scope.LoggedIn == true) {
                if (null != printPreviewDialog) {
                    $("#bootbox-modal-title").html("Label Printed Successfully");
                    $(".modal-header").css("background-color", "#5cb85c");

                    setTimeout(function () {
                        printPreviewDialog.hide();
                    }, 2000);
                }

                //                if ($toastrArray['print-label'] != null) {
                //                    $toastrArray['print-label'].attr('class', 'toast toast-success');
                //                    $toastrArray['print-label'].html(jobj['Text']);
                //                } else
                //                    toastr.success(jobj['Text'], "", {timeOut: 0});

                setTimeout(function () {
                    toastr.clear($toastrArray['print-label']);
                    $toastrArray['print-label'] = null;
                }, 2000);
            }
        } else {
            $history.prepend(moment(date).format('MMMM Do YYYY, h:mm:ss a') + ' : <span style="color:red">' + jobj['Text'] + '</span><br>');

            if ($scope.LoggedIn == true) {
                if (null != printPreviewDialog) {
                    $("#bootbox-modal-title").html("Label Print Error");
                    $(".modal-header").css("background-color", "#c9302c");
                    $(".bootbox-body").html('Print Failed: ' + jobj['Text']);
                    $("#bootbox-cancel-btn").show();
                }


                //                if ($toastrArray['print-label'] != null) {
                //                    $toastrArray['print-label'].attr('class', 'toast toast-error');
                //                    $toastrArray['print-label'].html('Print Failed: ' + jobj['Text'] + ' (This message will go away in 10 seconds)');
                //                } else
                //                    $toastrArray['print-label'] = toastr.error('Print Failed: ' + jobj['Text'] + ' (This message will go away in 10 seconds)', "", {timeOut: 0});
                //
                //                setTimeout(function () {
                //                    toastr.clear($toastrArray['print-label']);
                //                    $toastrArray['print-label'] = null;
                //                }, 10000);
            }
        }

        if (jobj['LastActiveStates'] != undefined) {
            $scope.Config.InputConfig.LastActiveStates = jobj['LastActiveStates'];
            $scope.$apply();
        }
    };



    $scope.barcodeScan = function (evt, jobj) {
        var $history = $("#history");

        var timestamp = jobj['Timestamp'];
        var date = new Date(timestamp);
        var text = jobj['Value'] + " was scanned";
        if (jobj['Match'] === 'true') {
            $history.prepend(moment(date).format('MMMM Do YYYY, h:mm:ss a') + ' : <span style="color:green">' + text + ' - Match</span><br>');
            //
            //            if ($toastrArray['print-label'] != null) {
            //                $toastrArray['print-label'].attr('class', 'toast toast-success');
            //                $toastrArray['print-label'].html(jobj['Text']);
            //            } else
            //                toastr.success(jobj['Value'] + " was scanned and was a match", "");
            //
            //            setTimeout(function() {
            //                toastr.clear($toastrArray['print-label']);
            //                $toastrArray['print-label'] = null;
            //            }, 2000);
        } else {
            $history.prepend(moment(date).format('MMMM Do YYYY, h:mm:ss a') + ' : <span style="color:red">' + text + ' - No Match</span><br>');
            //
            //            if ($toastrArray['print-label'] != null) {
            //                $toastrArray['print-label'].attr('class', 'toast toast-error');
            //                $toastrArray['print-label'].html('Print Failed: ' + jobj['Text'] + ' (This message will go away in 10 seconds)');
            //            } else
            //                $toastrArray['print-label'] = toastr.error('Print Failed: ' + jobj['Text'] + ' (This message will go away in 10 seconds)', "", {timeOut: 0});
            //
            //            setTimeout(function() {
            //                toastr.clear($toastrArray['print-label']);
            //                $toastrArray['print-label'] = null;
            //            }, 10000);
        }
    };


    var _labelWidth, _labelHeight;

    function labelLayout(labelConfig) {
        var $labelPreviewElem = $("#label-preview");
        _labelWidth = labelConfig.width;
        _labelHeight = labelConfig.height;

        var ratio = _labelWidth / _labelHeight;
        var width = 600;
        var height = 300;
        if (ratio < 2)
            width = height * ratio;
        else
            height = width / ratio;

        $labelPreviewElem.width(width);
        $labelPreviewElem.height(height);
    }



    $scope.getItemStyle = function (item) {
        if (null != item) {
            return {
                //background: item.selected ? "#ccc" : "",
                outline: item.selected ? "1px solid red" : "",
                position: "absolute",
                left: $scope.getX(item.xPos) + "px",
                top: $scope.getY(item.yPos) + "px",

                '-ms-transform': 'rotate(' + (item.rotate * 90) + 'deg)',  /* IE 9 */
                '-webkit-transform': 'rotate(' + (item.rotate * 90) + 'deg)', /* Safari */
                transform: 'rotate(' + (item.rotate * 90) + 'deg)',
                'transform-origin': 'top left'
            };
        }
    };



    $scope.getTextItemStyle = function (textItem) {
        var textSize = textItem.fontSize * .35 / 25.4;
        textSize = $scope.getY(textSize);

        var fontFamily = "monospace";
        if (textItem.font == "serif")
            fontFamily = "serif";
        else if (textItem.font == "sans-serif")
            fontFamily = "sans-serif";

        var fontStyle = "";
        if (textItem.fontStyle == 1)
            fontStyle = "font-weight: bold;";
        else if (textItem.fontStyle == 2)
            fontStyle = "font-style: italic;";
        else if (textItem.fontStyle == 3)
            fontStyle = "font-weight: bold; font-style: italic;";

        return {
            fontFamily: fontFamily,
            fontSize: textSize + "px",
            'white-space': nowrap
        };
    };



    $scope.getLabelStyle = function (labelConfig) {
        _labelHeight = labelConfig.height;
        _labelWidth = labelConfig.width;

        var ratio = _labelWidth / _labelHeight;
        var width = 600;
        var height = 305;
        if (ratio < 2)
            width = height * ratio;
        else
            height = width / ratio;

        return { width: width + "px", height: height + "px" };
    };



    $scope.getX = function (xInches) {
        var $labelPreviewElem = $("#label-preview");
        var fullWidth = $labelPreviewElem.width();
        var dpi = fullWidth / _labelWidth;
        return xInches * dpi;
    };



    $scope.getY = function (yInches) {
        var $labelPreviewElem = $("#label-preview");
        var fullHeight = $labelPreviewElem.height();
        var dpi = fullHeight / _labelHeight;
        return yInches * dpi;
    };



    $scope.getText = function (textItem) {
        if (textItem != undefined) {
            var text = textItem.value
            var newText = "";
            var offset = 0;
            var start = text.indexOf('{{', offset);
            if (start != 0)
                newText += text.substring(0, start);
            var end = text.indexOf('}}', offset);
            var lastProcessedIndex = 0;
            while (start >= 0 && end > 0 && end > start) {
                var variable = text.substring(start + 2, end);
                var variableType = variable.substring(0, variable.indexOf(':'));
                var variableFormat = variable.substring(variable.indexOf(':') + 1);

                if (variableFormat == 'xyzdate') {
                    var date = new Date();
                    newText += (date.getYear() % 10);
                    var month = date.getMonth() + 1;
                    if (month < 10)
                        newText += month;
                    else
                        newText += (month - 10 + 'X');
                    var day = date.getDate();
                    if (day < 10)
                        newText += '0' + day;
                    else
                        newText += day;

                } else if (variableFormat == 'juliandate') {
                    var date = new Date();
                    newText += (date.getYear() % 100);

                    var now = new Date();
                    var start = new Date(now.getFullYear(), 0, 0);
                    var diff = now - start;
                    var oneDay = 1000 * 60 * 60 * 24;
                    var day = Math.floor(diff / oneDay);
                    newText += day;

                } else if (variableType == 'date') {
                    var now = new Date();
                    variableFormat = variableFormat.replaceAll('yyyy', pad(now.getFullYear(), 2));
                    variableFormat = variableFormat.replaceAll('yy', pad(now.getYear(), 2));
                    variableFormat = variableFormat.replaceAll('MM', pad(now.getMonth() + 1, 2));
                    variableFormat = variableFormat.replaceAll('dd', pad(now.getDate(), 2));
                    newText += variableFormat;

                } else {
                    newText += variableFormat;
                }

                offset = end + 2;
                start = text.indexOf('{{', offset);

                if (start > end + 2) {
                    newText += text.substring(end + 2, start);
                }

                lastProcessedIndex = end + 2;
                end = text.indexOf('}}', offset);
            }
            if (lastProcessedIndex != text.length)
                newText += text.substring(lastProcessedIndex);
            if (newText === "")
                newText = text;
            return newText;
        }
    };



    function pad(n, width, z) {
        z = z || '0';
        n = n + '';
        return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
    }



    function showText(textItem) {
        var x = getX(textItem.xPos);
        var y = getY(textItem.yPos);

        var newText = $scope.getText(textItem);

        var textSize = textItem.fontSize * .35 / 25.4;
        textSize = getY(textSize);

        var fontFamily = "monospace";
        if (textItem.font == "serif")
            fontFamily = "serif";
        else if (textItem.font == "sans-serif")
            fontFamily = "sans-serif";

        var fontStyle = "";
        if (textItem.fontStyle == 1)
            fontStyle = "font-weight: bold;";
        else if (textItem.fontStyle == 2)
            fontStyle = "font-style: italic;";
        else if (textItem.fontStyle == 3)
            fontStyle = "font-weight: bold; font-style: italic;";

        var $div = $("<div style='position:absolute; top:" + y + "; font-family:" + fontFamily + "; font-size:" + textSize + "px; line-height:" + textSize + "px;" + fontStyle + "' ng-model=\"textItem\" ng-click=\"alert('text');\" ng-style=\"" + textItem.selected + " ? {\"background\"='#ccc'} : {'background'='#ccc'}\">" + newText + "</div>");
        var $labelPreviewElem = $("#label-preview");
        $labelPreviewElem.append($div);

        if (textItem.justification == "right") {
            var divWidth = $div.width();
            x -= divWidth;
        } else if (textItem.justification == "center") {
            var divWidth = $div.width();
            x -= (divWidth / 2.0);
        }
        $div.css({ left: x });
    }



    function showBarcode(barcodeItem) {
        var x = getX(barcodeItem.xPos);
        var y = getY(barcodeItem.yPos);
        var width = getX(barcodeItem.width);
        var height = getY(barcodeItem.height);
        var $barcodeElem = $("<div style='position:absolute; left:" + x + "; top:" + y + "; height:" + height + "px;'></div>");
        var $labelPreviewElem = $("#label-preview");
        $labelPreviewElem.append($barcodeElem);
        $barcodeElem.append(DrawHTMLBarcode_Code39($scope.getText(barcodeItem), 0, "yes", "px", parseInt(barcodeItem.narrowBarDots), 1, height, 3, "bottom", "left", "", "black", "white"));
    }


    $scope.showBarcode = function (elem, barcodeItem) {
        alert(elem);
        var x = $scope.getX(barcodeItem.xPos);
        var y = $scope.getY(barcodeItem.yPos);
        var height = $scope.getY(barcodeItem.height);
        var $barcodeElem = $("<div style='position:absolute; left:" + x + "; top:" + y + "; height:" + height + "px;'></div>");
        var $labelPreviewElem = $("#label-preview");
        $labelPreviewElem.append($barcodeElem);
        $barcodeElem.append(DrawHTMLBarcode_Code39($scope.getText(barcodeItem), 0, "yes", "px", parseInt(barcodeItem.narrowBarDots), 1, height, 3, "bottom", "left", "", "black", "white"));
    };


    function show2dBarcode(barcodeItem) {
        var x = getX(barcodeItem.xPos);
        var y = getY(barcodeItem.yPos);
        var size = getX(barcodeItem.size);
        var $barcodeElem = $("<div style='position:absolute; left:" + x + "; top:" + y + "; size:" + size + "px;'></div>");
        var $labelPreviewElem = $("#label-preview");
        $labelPreviewElem.append($barcodeElem);

        if (0 <= size) {
            var qrcode = new QRCode($barcodeElem[0], {
                text: $scope.getText(barcodeItem), //"i am jnior.  i am awesome!",
                width: size,
                height: size,
                correctLevel: QRCode.CorrectLevel.H
            });
        }
    }


    $scope.show2dBarcode = function ($elem, barcodeItem) {
        // alert($elem);
        var x = $scope.getX(barcodeItem.xPos);
        var y = $scope.getY(barcodeItem.yPos);
        var size = $scope.getX(barcodeItem.size);
        var $barcodeElem = $("<div style='position:absolute; left:" + x + "; top:" + y + "; size:" + size + "px;'></div>");
        $elem.append($barcodeElem);

        if (0 <= size) {
            var qrcode = new QRCode($barcodeElem[0], {
                text: $scope.getText(barcodeItem), //"i am jnior.  i am awesome!",
                width: size,
                height: size,
                correctLevel: QRCode.CorrectLevel.H
            });
        }
    };


    $scope.getCurrentInputStatesStyle = function (channel) {
        if ((($scope.CurrentInputStates >> (channel - 1)) & 1) == 1)
            return { 'background-color': '#66FF66' };
        else
            return { 'background-color': 'white' };
    };



    $scope.getInputStyle = function (channel) {
        var active = true;
        if ($scope.Config.InputConfig.Multiplexed == true) {
            if (channel == $scope.Config.InputConfig.PulseChannel)
                return { 'background-color': '#66FFFF' };
            if (channel < $scope.Config.InputConfig.StartChannel)
                active = false;
            else if (channel > $scope.Config.InputConfig.EndChannel)
                active = false;
        }

        if (!active)
            return { 'background-color': 'white', 'color': '#ccc' };
        if ((($scope.Config.InputConfig.LastActiveStates >> (channel - 1)) & 1) == 1)
            return { 'background-color': '#66FF66' };
        else
            return { 'background-color': 'white' };
    };





    $scope.hasChanged = function (config, savedConfig) {
        var changed = !angular.equals(config, savedConfig);
        return changed;
    };
}]);



app.directive('labelText', function () {
    return {
        restrict: 'E',
        scope: {
            barcodeItem: '='
        },
        transclude: true,
        link: function ($scope, $element, $attr) {
            $scope.$watch('barcodeItem', function (textItem) {
                var x = $scope.$parent.getX(textItem.xPos);
                var y = $scope.$parent.getY(textItem.yPos);
                var newText = $scope.$parent.getText(textItem);

                var textSize = textItem.fontSize * .35 / 25.4;
                textSize = $scope.$parent.getY(textSize);

                var fontFamily = "monospace";
                if (textItem.font == "serif")
                    fontFamily = "serif";
                else if (textItem.font == "sans-serif")
                    fontFamily = "sans-serif";

                var fontStyle = "";
                if (textItem.fontStyle == 1)
                    fontStyle = "font-weight: bold;";
                else if (textItem.fontStyle == 2)
                    fontStyle = "font-style: italic;";
                else if (textItem.fontStyle == 3)
                    fontStyle = "font-weight: bold; font-style: italic;";

                var $div = $("<div style='font-family:" + fontFamily + "; font-size:" + textSize + "px; line-height:" + textSize + "px;" + fontStyle + ";white-space: nowrap;' ng-model=\"textItem\">" + newText + "</div>");
                var $labelPreviewElem = $("#label-preview");
                $labelPreviewElem.append($div);

                if (textItem.justification == "right") {
                    var divWidth = $div.width();
                    x -= divWidth;
                } else if (textItem.justification == "center") {
                    var divWidth = $div.width();
                    x -= (divWidth / 2.0);
                }
                $div.css({ left: x });

                $element.empty();
                $element.append($div);

                // var height = $scope.$parent.getY(barcodeItem.height);
                // var $barcodeElem = $("<div style='height:" + height + "px;'></div>");

                // $barcodeElem.css('-ms-transform', 'rotate(' + (barcodeItem.rotate * 90) + 'deg)');  /* IE 9 */
                // $barcodeElem.css('-webkit-transform', 'rotate(' + (barcodeItem.rotate * 90) + 'deg)'); /* Safari */
                // $barcodeElem.css('transform', 'rotate(' + (barcodeItem.rotate * 90) + 'deg)');

                // $element.empty();
                // $element.append($barcodeElem);
                // // $barcodeElem.append(DrawHTMLBarcode_Code39($scope.$parent.getText(barcodeItem), 0, "yes", "px", parseInt(barcodeItem.narrowBarDots), 1, height, 3, "bottom", "left", "", "black", "white"));
                // $barcodeElem.append(DrawHTMLBarcode_Code39($scope.$parent.getText(barcodeItem), 0, "yes", "px", 1, 1, height, 3, "bottom", "left", "", "black", "white"));
            }, true);
        }
    };
});



app.directive('barcode', function () {
    return {
        restrict: 'E',
        scope: {
            barcodeItem: '='
        },
        transclude: true,
        link: function ($scope, $element, $attr) {
            $scope.$watch('barcodeItem', function (barcodeItem) {
                var height = $scope.$parent.getY(barcodeItem.height);
                var $barcodeElem = $("<div style='height:" + height + "px;white-space: nowrap;'></div>");

                $element.empty();
                $element.append($barcodeElem);
                // $barcodeElem.append(DrawHTMLBarcode_Code39($scope.$parent.getText(barcodeItem), 0, "yes", "px", parseInt(barcodeItem.narrowBarDots), 1, height, 3, "bottom", "left", "", "black", "white"));
                $barcodeElem.append(DrawHTMLBarcode_Code39($scope.$parent.getText(barcodeItem), 0, "yes", "px", 1, 1, height, 3, "bottom", "left", "", "black", "white"));
            }, true);
        }
    };
});



app.directive('barcode2d', function () {
    return {
        restrict: 'E',
        scope: {
            barcodeItem: '='
        },
        transclude: true,
        link: function ($scope, $element, $attr) {
            $scope.$watch('barcodeItem', function (barcodeItem) {
                var x = $scope.$parent.getX(barcodeItem.xPos);
                var y = $scope.$parent.getY(barcodeItem.yPos);
                var size = $scope.$parent.getX(barcodeItem.size);
                var $barcodeElem = $("<div style=''></div>");

                $element.empty();
                $element.append($barcodeElem);

                if (0 <= size) {
                    size *= .1;
                    var qrcode = new QRCode($barcodeElem[0], {
                        text: $scope.$parent.getText(barcodeItem),
                        width: size,
                        height: size,
                        correctLevel: QRCode.CorrectLevel.H
                    });
                }
            }, true);
        }
    };
});